/*
 * irp_instance.cpp
 *
 *  Created on: 26 de jul de 2016
 *      Author: hokama
 */

#include "irp2d_instance.h"

namespace IRP2D{
Irp2dInstance::Irp2dInstance():instance_type("mmirp2d"),verbose(1) {
}

Irp2dInstance::~Irp2dInstance() {
}

//This reads the instance in the specified file format
istream &operator>>(istream &input, Irp2dInstance &inst){
  if(inst.get_instance_type().compare("mmirp2d") != 0){
    cerr << "instance type not accepted." << endl;
  }
  input >> inst.number_of_customers >> inst.number_of_products  >> inst.horizon;

  inst.distance.resize(inst.number_of_customers+1);
  inst.edge_id.resize(inst.number_of_customers+1);
  for (int i=0; i < inst.number_of_customers+1; i++){
    inst.distance[i].resize(inst.number_of_customers+1);
    inst.edge_id[i].resize(inst.number_of_customers+1);
  }

  inst.x.resize(inst.number_of_customers+1);
  inst.y.resize(inst.number_of_customers+1);
  inst.max_area_customer.resize(inst.number_of_customers+1);

  inst.width.resize(inst.number_of_products);
  inst.height.resize(inst.number_of_products);

  inst.demand_of_product_per_customer_per_period.resize(inst.number_of_customers+2);
  for (int i=0; i < inst.number_of_customers+2; i++){
    inst.demand_of_product_per_customer_per_period[i].resize(inst.number_of_products);
    for (int l=0; l<inst.number_of_products; ++l)
      inst.demand_of_product_per_customer_per_period[i][l].resize(inst.horizon+2);
  }

  // ----------------------------------------------
  if(inst.get_verbose() >= 3) cout << "Memory allocation done " << endl;
  input >> inst.x[0];
  input >> inst.y[0];

  input >> inst.W;
  input >> inst.H;

  for (int l=0; l<inst.number_of_products; ++l){
    input >> inst.width[l] >> inst.width[l]; //ignore the product index
    input >> inst.height[l];
  }

  for (int i=1; i < inst.number_of_customers+1; i++){
    input >> inst.x[i] >> inst.x[i]; //ignore the customer index
    input >> inst.y[i];

    for (int l=0; l<inst.number_of_products; ++l){
      for (int t=1; t<inst.horizon+1; ++t)
        input >> inst.demand_of_product_per_customer_per_period[i][l][t];
    }

    for (int l=0; l<inst.number_of_products; ++l){
      inst.demand_of_product_per_customer_per_period[i][l][0] = 0;
      inst.demand_of_product_per_customer_per_period[i][l][inst.horizon+1] = 0;
    }
    input >> inst.max_area_customer[i];
  }

  int count = 0;
  for (int i = 0; i < inst.number_of_customers+1; i++){
    for (int j = i + 1; j < inst.number_of_customers+1; j++){
      inst.edge_id[i][j] = count;
      inst.edge_id[j][i] = count;
      count++;
    }
  }

  for (int i = 0; i < inst.number_of_customers+1; i++){
    for (int j = 0; j < inst.number_of_customers+1; j++){
      inst.distance[i][j] = (int) floor ( sqrt( pow (inst.x[i] - inst.x[j], 2) +
                                                pow (inst.y[i] - inst.y[j], 2)));
    }
  }

  input >> inst.number_of_vehicles;
  return input;
}

//This prints the instance in a human-readable format
ostream &operator<<( ostream &output, Irp2dInstance &inst){

  //block 1
  output << endl << "************************************Instance************************************" << endl << endl;
  output << "File path: " << inst.get_file_name() << endl;
  output << "Number of customers: " << inst.get_number_of_customers() << endl;
  output << "Number of products: " << inst.get_number_of_products() << endl;
  output << "Number of vehicles: " << inst.get_number_of_vehicles() << endl;
  output << "Length of planning horizon: " << inst.get_horizon() << endl;
  output << "Vehicle size: " << inst.get_vehicle_width() << "x" << inst.get_vehicle_height() << endl;

  output << "Coordinates: " << endl << "    i    x    y" << endl;
  for(int i = 0; i < inst.get_number_of_customers() + 1; i++){
    output.width(5);
    output << i;
    output.width(5);
    output << inst.get_x(i);
    output.width(5);
    output << inst.get_y(i);
    output << endl;
  }

  //block 2
  output << endl;
  if (getenv("TERM") != NULL) output << "\033[1;32m";
  output << "Distances: " << endl;
  if (getenv("TERM") != NULL) output << "\033[4;32m";
  output.width(5);
  output << "i/j";
  for(int i = 0; i < inst.get_number_of_customers() + 1; i++){
    output.width(5);
    output << i;
  }
  output << endl;
  if (getenv("TERM") != NULL) output << "\033[0m";
  for(int i = 0; i < inst.get_number_of_customers() + 1; i++){
    if (getenv("TERM") != NULL) output << "\033[1;32m";
    output.width(3);
    output << i << " |";
    if (getenv("TERM") != NULL) output << "\033[0m";
    for(int j = 0; j < inst.get_number_of_customers() + 1; j++){
      output.width(5);
      output << inst.get_distance(i, j);
    }
    output << endl;
  }

  //block 3
  output << endl;


  if (getenv("TERM") != NULL) output << "\033[1;32m";
  output << "Products: " << endl;
  output << "prod |       w |       h " << endl;
  if (getenv("TERM") != NULL) output << "\033[0m";

  for(int prod = 0; prod < inst.get_number_of_products(); prod++){
    output.width(4);
    output << prod;
    output.width(10);
    output << inst.get_width(prod);
    output.width(10);
    output << inst.get_height(prod) << "  ";
    output << endl;
  }

  //block 4
  output << endl;
  for(int customer = 1; customer < inst.get_number_of_customers() + 1; customer++){
    if (getenv("TERM") != NULL) output << "\033[1;32m";
    output << "At the customer " << customer << " with max area capacity (" << inst.get_max_area_customer(customer) << "): " << endl;
    output << "prod |   demand in period" << endl;
    if (getenv("TERM") != NULL) output << "\033[4;32m";
    output << "     |";
    for (int period = 1; period <= inst.get_horizon(); period++){
      output.width(6);
      output << period;
    }
    output  << endl;
    if (getenv("TERM") != NULL) output << "\033[0m";

    for(int prod = 0; prod < inst.get_number_of_products(); prod++){
      output.width(4);
      output << prod << "  ";
      output.width(10);
      for (int period = 1; period <= inst.get_horizon(); period++){
        output.width(6);
        output << inst.get_demand_of_product_per_customer_per_period(customer, prod, period);
      }
      output << endl;
    }
    output << endl;
  }

  output << "********************************************************************************" << endl << endl;
  return output;
}

} //end of namespace
